@@ -1,5 +1,16 @@
 #include "mainwindow.h"
-#include "ui_mainwindow.h"
+
+#ifdef Q_WS_MAEMO_5
+    #include <QtGui/QX11Info>
+    #include <X11/Xlib.h>
+    #include <X11/Xatom.h>
+    #include <QMaemo5InformationBox>
+
+    #include "pickselectoraction.h"
+#endif
+
+#include <QDebug>
+
 #include "fullscreenexitbutton.h"
 
 #include "gotodlg.h"
@@ -7,36 +18,37 @@
 #include "bookmarksdlg.h"
 #include "bookmarkdlg.h"
 
-#ifdef Q_WS_MAEMO_5
-    #include <QtGui/QX11Info>
-    #include <X11/Xlib.h>
-    #include <X11/Xatom.h>
-    #include <QMaemo5InformationBox>
-#endif
-
-#include <QDebug>
-
 #define SETTINGS_VERSION 1
 
-MainWindow::MainWindow(QWidget *parent) :
-    QMainWindow(parent),
-    m_pUI(new Ui::MainWindow)
+MainWindow::MainWindow(MainWindow *pParent) :
+    QMainWindow(),
+    // m_pUI(new Ui::MainWindow),
+    m_webPage(this)
 {
-    m_pUI->setupUi(this);
+    setWindowTitle("Macuco");
+    m_pParent = pParent;
+    // m_pUI->setupUi(this);
 
-    /*
-    m_zoomLevels << 30 << 50 << 67 << 80 << 90;
-    m_zoomLevels << 100;
-    m_zoomLevels << 110 << 120 << 133 << 150 << 170 << 200 << 240 << 300;
-    m_currentZoom = 100;
-    */
-
-    m_zoomFactor = 1;
+#ifdef WEBGRAPHICS
+    m_pGraphicsView = new WebGraphicsView();
+    m_pView = new QGraphicsWebView();
+    m_pGraphicsView->viewportItem()->setWidget(m_pView);
+    setCentralWidget(m_pGraphicsView);
+#else
+    m_pView = new WebView(this);
+    setCentralWidget(m_pView);
+#endif
 
     // loading label...
     m_pLoadingLabel = new QLabel(this);
     m_pLoadingLabel->setPixmap( QPixmap(QString::fromUtf8(":/images/spinner.png")) );
 
+    menuBar()->addAction("Home", this, SLOT(on_actionHome_triggered()));
+    menuBar()->addAction("Go To...", this, SLOT(on_actionGo_To_triggered()));
+    menuBar()->addAction("Bookmarks", this, SLOT(on_actionBookmarks_triggered()));
+    menuBar()->addAction("Add Bookmark", this, SLOT(on_actionAdd_Bookmark_triggered()));
+
+
 #ifdef Q_WS_MAEMO_5
     FullScreenExitButton *pButt = new FullScreenExitButton(this);
     connect(pButt, SIGNAL(clicked()), this, SLOT(showNormal()));
@@ -45,8 +57,19 @@ MainWindow::MainWindow(QWidget *parent) :
 
     setAttribute(Qt::WA_Maemo5StackedWindow);
 
-    m_pUI->actionFullscreen->setCheckable(false);
-    connect(m_pUI->actionFullscreen, SIGNAL(triggered()), this, SLOT(fullscreen_triggered()));
+    menuBar()->addAction("Fullscreen", this, SLOT(fullscreen_triggered()));
+
+    m_pOrientationSelector = new PickSelectorAction(this);
+    m_pOrientationSelector->setText("Orientation");
+    m_pOrientationSelector->items() << "Auto" << "Landscape" << "Portrait";
+    connect(m_pOrientationSelector, SIGNAL(selectionChanged(int)), this, SLOT(orientation_selectionChanged(int)));
+    menuBar()->addAction(m_pOrientationSelector);
+
+    m_pUserAgentSelector = new PickSelectorAction(this);
+    m_pUserAgentSelector->setText("User Agent");
+    m_pUserAgentSelector->items() << "IPhone" << "Default";
+    connect(m_pUserAgentSelector, SIGNAL(selectionChanged(int)), this, SLOT(userAgent_selectionChanged(int)));
+    menuBar()->addAction(m_pUserAgentSelector);
 
 #else // !Q_WS_MAEMO_5
     // error box
@@ -64,22 +87,46 @@ MainWindow::MainWindow(QWidget *parent) :
     connect(m_pMiniToolbar->getShowToolbarButton(), SIGNAL(clicked()), this, SLOT(showToolbar()));
     connect(m_pToolbar->getHideToolbarButton(), SIGNAL(clicked()), this, SLOT(hideToolbar()));
 
-    connect(m_pToolbar->getBackButton(), SIGNAL(clicked()), m_pUI->webView, SLOT(back()));
-    connect(m_pToolbar->getFowardButton(), SIGNAL(clicked()), m_pUI->webView, SLOT(forward()));
-    connect(m_pToolbar->getReloadButton(), SIGNAL(clicked()), m_pUI->webView, SLOT(reload()));
+    connect(m_pToolbar->getBackButton(), SIGNAL(clicked()), m_pView, SLOT(back()));
+    connect(m_pToolbar->getFowardButton(), SIGNAL(clicked()), m_pView, SLOT(forward()));
+    connect(m_pToolbar->getReloadButton(), SIGNAL(clicked()), m_pView, SLOT(reload()));
 
     connect(m_pToolbar, SIGNAL(urlChanged(QString)), this, SLOT(visitUrl(QString)));
 
-    m_pUI->actionFullscreen->setCheckable(true);
-    connect(m_pUI->actionFullscreen, SIGNAL(toggled(bool)), this, SLOT(setFullScreen(bool)));
+    QAction *pFullscreen = menuBar()->addAction("Fullscreen");
+    pFullscreen->setCheckable(true);
+    connect(pFullscreen, SIGNAL(toggled(bool)), this, SLOT(setFullScreen(bool)));
+
+    // user agent
+    // ...
 #endif
 
-    loadSettings();
+    if ( m_pParent ) {
+        // copy settings from mainwindow
+        m_zoomFactor = m_pParent->m_zoomFactor;
+        m_pBookmarks = m_pParent->m_pBookmarks;
+        setFullScreen(m_pParent->isFullScreen());
+        setUserAgent( m_pParent->m_webPage.isIphoneUserAgent() );
 
+#ifdef Q_WS_MAEMO_5
+        qDebug() << "New Window: " << m_pParent->isLandscape() << " - " << m_pParent->isAutoRotate();
+        setAutoRotate(false);
+        setLandscape(m_pParent->isLandscape());
+#endif
 
+    } else {
+        m_zoomFactor = 2;
+        m_pBookmarks = new QList<Bookmark>();
+
+        // load settings
+        loadSettings();
+    }
+
+#ifndef WEBGRAPHICS
     // set web page
     m_webPage.mainFrame()->setScrollBarPolicy(Qt::Vertical, Qt::ScrollBarAlwaysOff);
     m_webPage.mainFrame()->setScrollBarPolicy(Qt::Horizontal, Qt::ScrollBarAlwaysOff);
+#endif
 
     connect(m_webPage.networkAccessManager(), SIGNAL(finished(QNetworkReply*)),
         this, SLOT(finishedRequest(QNetworkReply*)));
@@ -95,47 +142,96 @@ MainWindow::MainWindow(QWidget *parent) :
     // QWebSettings::enablePersistentStorage(path);
 
     // connect(&m_webPage, SIGNAL(linkClicked(QUrl)), this, SLOT(linkClicked(QUrl)));
-    // connect(&m_webPage, SIGNAL(unsupportedContent(QNetworkReply*)), this, SLOT(unsupportedContent(QNetworkReply*)));
+    connect(&m_webPage, SIGNAL(unsupportedContent(QNetworkReply*)), this, SLOT(unsupportedContent(QNetworkReply*)));
+    connect(&m_webPage, SIGNAL(downloadRequested(QNetworkRequest)), this, SLOT(downloadRequested(QNetworkRequest)));
 
-    // set webview
-    connect(m_pUI->webView, SIGNAL(loadStarted()), this, SLOT(showLoading()));
-    connect(m_pUI->webView, SIGNAL(urlChanged(QUrl)), this, SLOT(changeUrl(QUrl)));
-    connect(m_pUI->webView, SIGNAL(loadFinished(bool)), this, SLOT(hideLoading(bool)));
+    connect(m_pView, SIGNAL(loadStarted()), this, SLOT(showLoading()));
+    connect(m_pView, SIGNAL(urlChanged(QUrl)), this, SLOT(changeUrl(QUrl)));
+    connect(m_pView, SIGNAL(loadFinished(bool)), this, SLOT(hideLoading(bool)));
 
-    m_pUI->webView->setPage(&m_webPage);
+    m_pView->setPage(&m_webPage);
 
-    m_pUI->webView->setWindowTitle("Macuco");
-    on_actionHome_triggered();
+    if ( !m_pParent ) visitUrl("http://www.crochik.com/macuco");
 }
 
 MainWindow::~MainWindow()
 {
+    qDebug() << "MainWindow::~MainWindow";
+    if ( m_pParent ) {
+        qDebug() << ">> not top most";
+        m_pBookmarks = NULL;
+        return;
+    }
+
+    qDebug() << ">> top most window";
+    if ( m_pBookmarks ) {delete m_pBookmarks; m_pBookmarks = NULL;}
+
+    // delete children windows
+    while ( !m_children.isEmpty() ) delete m_children.takeLast();
+}
+
+void MainWindow::userAgent_selectionChanged(int index) {
+    switch ( index ) {
+        case 0: // iphone
+            m_webPage.setIPhoneUserAgent();
+            break;
+
+        case 1: // default
+            m_webPage.setIPhoneUserAgent(false);
+            break;
+    }
+}
+
+void MainWindow::orientation_selectionChanged(int index) {
+    switch ( index ) {
+        case 0: // auto
+            setAutoRotate(true);
+            break;
+
+        case 1: // landscape
+            setAutoRotate(false);
+            setLandscape(true);
+            break;
+
+        case 2: // portrait
+            setAutoRotate(false);
+            setLandscape(false);
+            break;
+    }
+}
+
+MainWindow* MainWindow::topMost() {
+    if ( m_pParent ) return m_pParent->topMost();
+    return this;
+}
+
+MainWindow* MainWindow::createWindow() {
+    if ( m_pParent ) return m_pParent->createWindow();
+
+    // top most
+    MainWindow *pChild =new MainWindow(this);
+    m_children.append(pChild);
+
+    return pChild;
 }
 
 void MainWindow::loadSettings() {
     Bookmark::registerMeataType();
 
     QSettings settings("Macuco", "Macuco");
-
     int version = settings.value("version", 0).toInt();
     if ( version<SETTINGS_VERSION ) {
         qDebug() << "previous version of settings file";
         //...
     }
 
-    m_bookmarks = settings.value("bookmarks").value< QList<Bookmark> >();
+    (*m_pBookmarks) = settings.value("bookmarks").value< QList<Bookmark> >();
     setFullScreen(settings.value("fullScreen", false).toBool());
     setUserAgent(settings.value("iphoneUserAgent", true).toBool());
 
-#ifndef Q_WS_MAEMO_5
-    m_pUI->actionLandscape->setVisible(false);
-    m_pUI->actionAuto_Rotate->setVisible(false);
-
-#else
-    setLandscape(settings.value("isLandscape", true).toBool());
-    // setAutoRotate(settings.value("autoRotate", true).toBool());
-    setAutoRotate(true);
-    m_pUI->actionAuto_Rotate->setVisible(false);
+#ifdef Q_WS_MAEMO_5
+    int orientation = settings.value("orientation", 0).toInt();
+    orientation_selectionChanged(orientation);
 #endif
 }
 
@@ -146,18 +242,12 @@ void MainWindow::saveSettings() {
     settings.setValue("iphoneUserAgent", m_webPage.isIphoneUserAgent());
 
 #ifdef Q_WS_MAEMO_5
-    settings.setValue("isLandscape", isLandscape());
-    settings.setValue("autoRotate", isAutoRotate());
+    settings.setValue("orientation", m_pOrientationSelector->selectedIndex());
 #endif
 
     QVariant bookmarks;
-    bookmarks.setValue(m_bookmarks);
+    bookmarks.setValue(*m_pBookmarks);
     settings.setValue("bookmarks", bookmarks);
-
-}
-
-QWebView* MainWindow::getWebView() {
-    return m_pUI->webView;
 }
 
 void MainWindow::showLoading() {
@@ -189,58 +279,15 @@ void MainWindow::changeUrl(QUrl url)
 #endif
 }
 
-void MainWindow::visitUrl(QString urlString)
+void MainWindow::visitUrl(QString url)
 {
-    m_pUI->webView->load(QUrl::fromUserInput(urlString));
-}
-
-void MainWindow::hideLoading(bool success) {
-    m_pLoadingLabel->hide();
-
-#ifdef Q_WS_MAEMO_5
-    setAttribute(Qt::WA_Maemo5ShowProgressIndicator, false);
-
-    // use notification
-    // ...
-
-    if ( !success ) {
-        QString msg("<b>Loading Error</b><br/>");
-        if ( m_errors.length()>1 ) msg+=m_errors.join("<br/>"); else msg+=m_currentUrl.toString();
-        QMaemo5InformationBox::information(this,
-            msg,
-            QMaemo5InformationBox::DefaultTimeout
-        );
-
-        /*
-        if ( QMessageBox::warning(
-            this,
-            "Loading Error",
-            m_errors.count()>0 ? m_errors.join("\n") : "Unspecified",
-            QMessageBox::Retry, QMessageBox::Cancel
-        )== QMessageBox::Retry )
-        {
-            // retry
-            reload();
-        }
-        */
-        return;
-    }
-
-#else
-    hideOverlayOnPage();
-
-    if (success) {
-        m_pErrorBox->hide();
-    } else {
-        showOverlayOnPage();
-        m_pErrorBox->show();
-    }
-#endif
+    if ( url.isNull() || url.isEmpty() ) return;
+    m_pView->load(QUrl::fromUserInput(url));
 }
 
 void MainWindow::reload()
 {
-    m_pUI->webView->load(m_currentUrl);
+    m_pView->load(m_currentUrl);
 
 #ifndef Q_WS_MAEMO_5
     m_pErrorBox->hide();
@@ -281,20 +328,15 @@ void MainWindow::closeEvent(QCloseEvent *event)
     grabZoomKeys(false);
 #endif
 
-    saveSettings();
+    if ( !m_pParent ) {
+        // top most
+        saveSettings();
+        m_webPage.saveCookies();
+    }
 
-    m_webPage.saveCookies();
     event->accept();
 }
 
-void MainWindow::finishedRequest(QNetworkReply* reply)
-{
-    if (reply->error() == QNetworkReply::NoError) return;
-
-    m_errors << reply->errorString();
-    qDebug() << "Error(s): " << m_errors;
-}
-
 void MainWindow::zoomIn() {
     setZoom(m_zoomFactor*1.15);
 }
@@ -304,11 +346,14 @@ void MainWindow::zoomOut() {
 }
 
 void MainWindow::setZoom(qreal zoomFactor) {
-    disableWebkitAutoSize();
+    enableWebkitAutoSize();
     m_zoomFactor = zoomFactor;
     if ( m_zoomFactor>.90 && m_zoomFactor<1.1) m_zoomFactor =1;
 
-    m_pUI->webView->setZoomFactor(m_zoomFactor);
+    m_pView->setZoomFactor(m_zoomFactor);
+
+    if ( m_pGraphicsView )
+        m_pGraphicsView->viewportItem()->resizeWebViewToFrame();
 }
 
 void MainWindow::keyPressEvent(QKeyEvent *event)
@@ -354,20 +399,22 @@ void MainWindow::keyPressEvent(QKeyEvent *event)
     }
 }
 
-void MainWindow::disableWebkitAutoSize()
+void MainWindow::enableWebkitAutoSize()
 {
-    m_pUI->webView->page()->mainFrame()->evaluateJavaScript("document.body.style.WebkitTextSizeAdjust = 'auto'");
+    m_pView->page()->mainFrame()->evaluateJavaScript("document.body.style.WebkitTextSizeAdjust = 'auto'");
 }
 
 void MainWindow::on_actionHome_triggered() {
+    m_zoomFactor = 1.5;
     visitUrl("http://www.crochik.com/macuco");
 }
 
 void MainWindow::on_actionBookmarks_triggered()
 {
-    BookmarksDlg dlg(m_bookmarks, this);
+    BookmarksDlg dlg((*m_pBookmarks), this);
 
 #ifdef Q_WS_MAEMO_5
+    if ( !isLandscape() ) dlg.setAttribute(Qt::WA_Maemo5PortraitOrientation);
     dlg.setAttribute(Qt::WA_Maemo5StackedWindow);
     dlg.setWindowFlags(Qt::Window);
 #endif
@@ -410,7 +457,7 @@ void MainWindow::on_actionAdd_Bookmark_triggered()
 {
     BookmarkDlg dlg(this);
     dlg.setUrl(m_currentUrl.toString());
-    dlg.setTitle(m_pUI->webView->title());
+    dlg.setTitle(m_pView->title());
     dlg.removeBtt()->setVisible(false);
 
     if ( dlg.exec()!= QDialog::Accepted ) return;
@@ -418,7 +465,7 @@ void MainWindow::on_actionAdd_Bookmark_triggered()
     Bookmark mark;
     mark.setTitle(dlg.title());
     mark.setUrl(dlg.url());
-    m_bookmarks.append(mark);
+    m_pBookmarks->append(mark);
 }
 
 void MainWindow::on_actionLandscape_triggered() {
@@ -443,16 +490,17 @@ void MainWindow::orientationChanged() {
     QRect screenGeometry = QApplication::desktop()->screenGeometry();
 
     qDebug() << "Orientation has changed: " << screenGeometry.width() <<", " << screenGeometry.height();
-    setLandscape( screenGeometry.width() > screenGeometry.height(), true );
+
+    // setLandscape( screenGeometry.width() > screenGeometry.height(), true );
 }
 
-void MainWindow::setLandscape(bool landscape/*=true*/, bool force/*=false*/) {
-    if ( !force && landscape==isLandscape() ) return;
+void MainWindow::setLandscape(bool landscape/*=true*/) {
+    m_pOrientationSelector->setSelectedIndex(landscape?1:2);
 
     if ( landscape ) {
         qDebug() << ">> set Landscape";
         //setAttribute(Qt::WA_Maemo5PortraitOrientation, false);
-        setAttribute(Qt::WA_Maemo5LandscapeOrientation, true);
+        setAttribute(Qt::WA_Maemo5LandscapeOrientation, true); //
 
     } else {
         qDebug() << ">> set Portrait";
@@ -466,22 +514,18 @@ void MainWindow::setAutoRotate(bool autoRotate /*=true*/) {
     // http://qt.nokia.com/doc/qt-maemo-4.6/maemo5-rotation.html
     if ( autoRotate ) {
         qDebug() << "set autorotate: on";
-        m_pUI->actionAuto_Rotate->setText("Auto Rotate");
         setAttribute(Qt::WA_Maemo5AutoOrientation, true);
-
-        connect(QApplication::desktop(), SIGNAL(resized(int)), this, SLOT(orientationChanged()));
+        m_pOrientationSelector->setSelectedIndex(0);
         return;
     }
 
     qDebug() << "set autorotate: off";
-    m_pUI->actionAuto_Rotate->setText("Disable Auto-Rotate");
     setAttribute(Qt::WA_Maemo5AutoOrientation, false);
-
-    disconnect(QApplication::desktop(), SIGNAL(resized(int)), this, SLOT(orientationChanged()));
 }
 
 bool MainWindow::isLandscape(){
-    return testAttribute(Qt::WA_Maemo5LandscapeOrientation) || !testAttribute(Qt::WA_Maemo5PortraitOrientation);
+    return width()>height();
+    // return testAttribute(Qt::WA_Maemo5LandscapeOrientation) || !testAttribute(Qt::WA_Maemo5PortraitOrientation);
 }
 
 bool MainWindow::isAutoRotate(){
@@ -560,7 +604,7 @@ void MainWindow::on_actionUser_Agent_triggered()
 
 void MainWindow::setUserAgent(bool iphone/*=true*/) {
     m_webPage.setIPhoneUserAgent(iphone);
-    m_pUI->actionUser_Agent->setText(iphone ? "Default User Agent" : "iPhone User Agent");
+    m_pUserAgentSelector->setSelectedIndex(iphone?0:1);
 }
 
 void MainWindow::linkClicked( const QUrl & url ) {
@@ -568,5 +612,77 @@ void MainWindow::linkClicked( const QUrl & url ) {
 }
 
 void MainWindow::unsupportedContent(QNetworkReply* pReply) {
-    qDebug() << "unsupportedContent: " << pReply;
+
+    qDebug() << "unsupportedContent: " << pReply->request().url(); // << pReply;
+    QList<QByteArray> headerList = pReply->rawHeaderList();
+    QString mimeType = pReply->header(QNetworkRequest::ContentTypeHeader).toString();
+    qDebug() << ">> mime type: " << mimeType;
+    foreach ( QByteArray array, headerList ) {
+        QString header(array);
+        QString value(pReply->rawHeader(array));
+        qDebug() << ">> Header: " << header << value;
+    }
+
+}
+
+void MainWindow::downloadRequested (const QNetworkRequest& request) {
+    qDebug() << "downloadRequested: "; // << request;
+}
+
+void MainWindow::finishedRequest(QNetworkReply* reply)
+{
+    if (reply->error() == QNetworkReply::NoError) return;
+
+    m_errors << reply->errorString();
+    qDebug() << "Error(s): " << m_errors;
+}
+
+void MainWindow::hideLoading(bool success) {
+    m_pLoadingLabel->hide();
+
+    setZoom(m_zoomFactor);
+
+#ifdef Q_WS_MAEMO_5
+    setAttribute(Qt::WA_Maemo5ShowProgressIndicator, false);
+
+    // use notification
+    // ...
+
+    if ( !success ) {
+        QString msg("<b>Loading Error</b><br/>");
+        if ( m_errors.length()>1 ) msg+=m_errors.join("<br/>"); else msg+=m_currentUrl.toString();
+        QMaemo5InformationBox::information(this,
+            msg,
+            QMaemo5InformationBox::DefaultTimeout
+        );
+
+        /*
+        if ( QMessageBox::warning(
+            this,
+            "Loading Error",
+            m_errors.count()>0 ? m_errors.join("\n") : "Unspecified",
+            QMessageBox::Retry, QMessageBox::Cancel
+        )== QMessageBox::Retry )
+        {
+            // retry
+            reload();
+        }
+        */
+        return;
+    }
+
+#else
+    hideOverlayOnPage();
+
+    if (success) {
+        m_pErrorBox->hide();
+    } else {
+        showOverlayOnPage();
+        m_pErrorBox->show();
+    }
+#endif
+}
+
+QWebPage *MainWindow::page() {
+    return &m_webPage;
 }
