@@ -2,15 +2,29 @@
 #define MAINWINDOW_H
 
 #include <QtGui/QMainWindow>
-#include "webview.h"
+#include <QLabel>
+
+// -----------------------------------------------------------------------
+#define WEBGRAPHICS
+// -----------------------------------------------------------------------
+
+#ifdef WEBGRAPHICS
+    #include <QGraphicsWebView>
+    #include "webgraphicsview.h"
+#else
+    #include "webview.h"
+#endif
+
+#ifdef Q_WS_MAEMO_5
+    #include "pickselectoraction.h"
+#endif
+
 #include "errorbox.h"
 #include "toolbar.h"
 #include "minifiedtoolbar.h"
 #include "bookmark.h"
 #include "webpage.h"
 
-#include <QLabel>
-
 namespace Ui {
     class MainWindow;
 }
@@ -19,15 +33,15 @@ class MainWindow : public QMainWindow {
     Q_OBJECT
 
 public:
-    explicit MainWindow(QWidget *parent = 0);
+    explicit MainWindow(MainWindow *pParent);
     virtual ~MainWindow();
 
-    //manually adapt layout to new size
-    void adjustSizes();
-
+    void adjustSizes(); //manually adapt layout to new size
     void keyPressEvent(QKeyEvent *event);
 
-    QWebView* getWebView();
+    QWebPage *page();
+    MainWindow *createWindow();
+    MainWindow *topMost();
 
 private:
     void loadSettings();
@@ -46,7 +60,7 @@ private:
     void zoomFit();
     void setZoom(qreal zoomFactor);
 
-    void disableWebkitAutoSize();
+    void enableWebkitAutoSize();
 
     void setUserAgent(bool iphone=true);
 
@@ -54,7 +68,7 @@ private:
     void grabZoomKeys(bool grab);
     bool isLandscape();
     bool isAutoRotate();
-    void setLandscape(bool landscape=true, bool force=false);
+    void setLandscape(bool landscape=true);
     void setAutoRotate(bool autoRotate=true);
 #else
     //white overlay behind the error message
@@ -82,6 +96,10 @@ private slots:
     // QWebView/QWebPage
     void linkClicked ( const QUrl & url );
     void unsupportedContent(QNetworkReply*);
+    void downloadRequested (const QNetworkRequest& request);
+
+    void userAgent_selectionChanged(int index);
+    void orientation_selectionChanged(int index);
 
 #ifdef Q_WS_MAEMO_5
     void orientationChanged();
@@ -99,16 +117,29 @@ private slots:
 
 private:
     // mainWindow components
-    QUrl m_currentUrl;
-    qreal m_zoomFactor;
-    Ui::MainWindow *m_pUI;
-    QLabel* m_pLoadingLabel;
-    QList<Bookmark> m_bookmarks;
-    WebPage m_webPage;
+    QUrl            m_currentUrl;
+    qreal           m_zoomFactor;
+    QLabel          *m_pLoadingLabel;
+    QStringList     m_errors;
 
-    QStringList m_errors;
+    QList<Bookmark> *m_pBookmarks;
+    WebPage         m_webPage;
+    MainWindow      *m_pParent;
+    QList<MainWindow *> m_children;
 
-#ifndef Q_WS_MAEMO_5
+#ifdef WEBGRAPHICS
+    // v2
+    QGraphicsWebView    *m_pView;
+    WebGraphicsView     *m_pGraphicsView;
+#else
+    // qt 4.5/not maemo
+    WebView             *m_pView;
+#endif
+
+#ifdef Q_WS_MAEMO_5
+    PickSelectorAction  *m_pUserAgentSelector;
+    PickSelectorAction  *m_pOrientationSelector;
+#else
     ErrorBox* m_pErrorBox;
     Toolbar* m_pToolbar;
     MinifiedToolbar* m_pMiniToolbar;
