@@ -317,12 +317,16 @@ void WebView::mouseReleaseEvent(QMouseEvent *pEvent) {
 
 QWebView* WebView::createWindow ( QWebPage::WebWindowType type )
 {
+#ifndef WEBGRAPHICS
     Q_UNUSED(type);
 
-    MainWindow* new_window = new MainWindow();
+    MainWindow* new_window = new MainWindow(QString::null);
     new_window->show();
 
     return new_window->getWebView();
+#else
+    return 0;
+#endif
 }
 
 static QPoint deaccelerate(const QPoint &speed, int a = 1, int max = 64)
